@@ -99,7 +99,10 @@ struct _GSSDPClientPrivate {
 
         gboolean           active;
         gboolean           initialized;
+        gint32             boot_id; /* "Non-negative 31 bit integer */
+        gint32             config_id; /* "Non-negative 31 bit integer, User-assignable from 0 - 2^24 -1 */
 };
+
 typedef struct _GSSDPClientPrivate GSSDPClientPrivate;
 
 G_DEFINE_TYPE_EXTENDED (GSSDPClient,
@@ -129,6 +132,8 @@ enum {
         PROP_MSEARCH_PORT,
         PROP_ADDRESS_FAMILY,
         PROP_UDA_VERSION,
+        PROP_BOOT_ID,
+        PROP_CONFIG_ID,
 };
 
 enum {
@@ -174,8 +179,6 @@ gssdp_client_init (GSSDPClient *client)
 
         priv->active = TRUE;
 
-        /* Generate default server ID */
-        priv->server_id = make_server_id (gssdp_client_get_uda_version (client));
 }
 
 static void
@@ -198,11 +201,27 @@ gssdp_client_initable_init (GInitable                   *initable,
         if (priv->initialized)
                 return TRUE;
 
+        /* The following two fall-backs are there to make things
+         * compatible with GSSDP 1.0.x and earlier */
+
         /* Fall-back to UDA 1.0 if no version is specified */
         if (priv->uda_version == GSSDP_UDA_VERSION_UNSPECIFIED) {
                 priv->uda_version = GSSDP_UDA_VERSION_1_0;
         }
 
+        /* Fall-back to IPv4 if no socket family was specified */
+        if (priv->device.address_family == G_SOCKET_FAMILY_INVALID) {
+                priv->device.address_family = G_SOCKET_FAMILY_IPV4;
+        }
+
+        /* Generate default server ID */
+        if (priv->server_id == NULL) {
+                GSSDPUDAVersion version;
+
+                version = gssdp_client_get_uda_version (client);
+                priv->server_id = make_server_id (version);
+        }
+
         if (!gssdp_net_init (error))
                 return FALSE;
 
@@ -333,6 +352,12 @@ gssdp_client_get_property (GObject    *object,
         case PROP_UDA_VERSION:
                 g_value_set_enum (value, priv->uda_version);
                 break;
+        case PROP_BOOT_ID:
+                g_value_set_int (value, priv->boot_id);
+                break;
+        case PROP_CONFIG_ID:
+                g_value_set_int (value, priv->config_id);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
                 break;
@@ -386,6 +411,12 @@ gssdp_client_set_property (GObject      *object,
         case PROP_UDA_VERSION:
                 priv->uda_version = g_value_get_enum (value);
                 break;
+        case PROP_BOOT_ID:
+                gssdp_client_set_boot_id (client, g_value_get_int (value));
+                break;
+        case PROP_CONFIG_ID:
+                gssdp_client_set_config_id (client, g_value_get_int (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
                 break;
@@ -642,6 +673,48 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                          G_PARAM_CONSTRUCT_ONLY |
                          G_PARAM_STATIC_STRINGS));
 
+        /**
+         * GSSDPClient:boot-id:
+         *
+         * The value of the BOOTID.UPNP.ORG header
+         *
+         * Since 1.1.2
+         */
+        g_object_class_install_property
+                (object_class,
+                 PROP_BOOT_ID,
+                 g_param_spec_int
+                        ("boot-id",
+                         "current boot-id value",
+                         "Value of the BOOTID.UPNP.ORG header",
+                         -1,
+                         G_MAXINT32,
+                         -1,
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT |
+                         G_PARAM_STATIC_STRINGS));
+
+        /**
+         * GSSDPClient:config-id:
+         *
+         * The value of the BOOTID.UPNP.ORG header
+         *
+         * Since 1.1.2
+         */
+        g_object_class_install_property
+                (object_class,
+                 PROP_CONFIG_ID,
+                 g_param_spec_int
+                        ("config-id",
+                         "current config-id value",
+                         "Value of the CONFIGID.UPNP.ORG header",
+                         -1,
+                         G_MAXINT32,
+                         -1,
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT |
+                         G_PARAM_STATIC_STRINGS));
+
         /**
          * GSSDPClient::message-received: (skip)
          * @client: The #GSSDPClient that received the message.
@@ -1015,12 +1088,25 @@ void
 gssdp_client_clear_headers (GSSDPClient *client)
 {
         GSSDPClientPrivate *priv = NULL;
+        GList *l;
 
         g_return_if_fail (GSSDP_IS_CLIENT (client));
         priv = gssdp_client_get_instance_private (client);
 
-        g_list_free_full (priv->headers,
-                          (GDestroyNotify) header_field_free);
+        l = priv->headers;
+        while (l != NULL)
+        {
+                GList *next = l->next;
+                GSSDPHeaderField *header = l->data;
+
+                if (g_strcmp0 (header->name, "BOOTID.UPNP.ORG") != 0 &&
+                    g_strcmp0 (header->name, "CONFIGID.UPNP.ORG") != 0) {
+                        header_field_free (header);
+                        priv->headers = g_list_delete_link (priv->headers, l);
+                }
+                l = next;
+        }
+
 }
 
 /**
@@ -1053,6 +1139,13 @@ gssdp_client_get_index (GSSDPClient *client)
         return priv->device.index;
 }
 
+/**
+ * gssdp_client_get_family:
+ * @client: A #GSSDPClient
+ *
+ * Returns: IP protocol version (%G_SOCKET_FAMILY_IPV4 or G_SOCKET_FAMILY_IPV6)
+ * this client uses
+ */
 GSocketFamily
 gssdp_client_get_family (GSSDPClient *client)
 {
@@ -1062,6 +1155,12 @@ gssdp_client_get_family (GSSDPClient *client)
         return g_inet_address_get_family (priv->device.host_addr);
 }
 
+/**
+ * gssdp_client_get_uda_version:
+ * @client: A #GSSDPClient
+ *
+ * Returns: the UDA protocol version this client adheres to
+ */
 GSSDPUDAVersion
 gssdp_client_get_uda_version  (GSSDPClient *client)
 {
@@ -1071,6 +1170,65 @@ gssdp_client_get_uda_version  (GSSDPClient *client)
         return priv->uda_version;
 }
 
+/**
+ * gssdp_client_set_boot_id:
+ * @client: A #GSSDPClient
+ * @boot_id: The new boot-id for the client
+ *
+ * Will set the new boot-id for this SSDP client. Does nothing if the UDA
+ * version used by the client is UDA 1.0
+ *
+ * The boot-id is used to signalize changes in the network configuration
+ * for multi-homed hosts
+ */
+void
+gssdp_client_set_boot_id (GSSDPClient *client, gint32 boot_id)
+{
+
+        g_return_if_fail (GSSDP_IS_CLIENT (client));
+
+        GSSDPClientPrivate *priv = gssdp_client_get_instance_private (client);
+
+        priv->boot_id = boot_id;
+
+        if (priv->uda_version > GSSDP_UDA_VERSION_1_0) {
+                char *id_string;
+                gssdp_client_remove_header (client, "BOOTID.UPNP.ORG");
+
+                id_string = g_strdup_printf ("%d", boot_id);
+                gssdp_client_append_header (client, "BOOTID.UPNP.ORG", id_string);
+                g_free (id_string);
+        }
+
+}
+
+/**
+ * gssdp_client_set_config_id:
+ * @client: A #GSSDPClient
+ * @config_id: The new config-id for the client
+ *
+ * The config-id is used to allow caching of the device or service description.
+ * It should be changed if that changes.
+ */
+void
+gssdp_client_set_config_id (GSSDPClient *client, gint32 config_id)
+{
+        g_return_if_fail (GSSDP_IS_CLIENT (client));
+
+        GSSDPClientPrivate *priv = gssdp_client_get_instance_private (client);
+
+        if (priv->uda_version > GSSDP_UDA_VERSION_1_0) {
+                char *id_string;
+                priv->config_id = config_id;
+                gssdp_client_remove_header (client, "CONFIGID.UPNP.ORG");
+
+                id_string = g_strdup_printf ("%d", config_id);
+                gssdp_client_append_header (client, "CONFIGID.UPNP.ORG", id_string);
+                g_free (id_string);
+        }
+
+}
+
 /**
  * _gssdp_client_send_message:
  * @client: A #GSSDPClient
@@ -1164,6 +1322,15 @@ _gssdp_client_get_mcast_group (GSSDPClient *client)
         }
 }
 
+#define ENSURE_V6_GROUP(group) \
+        G_STMT_START { \
+                if (SSDP_V6_ ## group ## _ADDR == NULL) { \
+                        SSDP_V6_ ## group ## _ADDR = \
+                                g_inet_address_new_from_string (SSDP_V6_ ## group); \
+                } \
+        } \
+        G_STMT_END
+
 static GInetAddress *
 _gssdp_client_get_mcast_group_addr (GSSDPClient *client)
 {
@@ -1182,15 +1349,11 @@ _gssdp_client_get_mcast_group_addr (GSSDPClient *client)
                 /* According to Annex.A, we need to check the scope of the
                  * address to use the proper multicast group */
                 if (g_inet_address_get_is_link_local (priv->device.host_addr)) {
-                            if (SSDP_V6_LL_ADDR == NULL) {
-                                    SSDP_V6_LL_ADDR = g_inet_address_new_from_string (SSDP_V6_LL);
-                            }
+                            ENSURE_V6_GROUP(LL);
 
                             return SSDP_V6_LL_ADDR;
                 } else {
-                            if (SSDP_V6_SL_ADDR == NULL) {
-                                    SSDP_V6_SL_ADDR = g_inet_address_new_from_string (SSDP_V6_SL);
-                            }
+                            ENSURE_V6_GROUP(SL);
 
                             return SSDP_V6_SL_ADDR;
                 }
